@@ -150,8 +150,11 @@ gui_main_get_password (const char **prompt, char *password, int size)
  */
 
 void
-gui_main_signal_sigint ()
+gui_main_signal_sigint (int signo)
 {
+    /* make C compiler happy */
+    (void) signo;
+
     weechat_quit = 1;
 }
 
@@ -253,8 +256,11 @@ gui_main_init ()
  */
 
 void
-gui_main_signal_sigwinch ()
+gui_main_signal_sigwinch (int signo)
 {
+    /* make C compiler happy */
+    (void) signo;
+
     gui_signal_sigwinch_received = 1;
 }
 
