@@ -1,7 +1,7 @@
 /*
  * buflist-bar-item.c - bar item for buflist plugin
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -74,6 +74,9 @@ buflist_bar_item_get_index (const char *item_name)
     int i;
     const char *ptr_item_name;
 
+    if (!item_name)
+        return -1;
+
     for (i = 0; i < BUFLIST_BAR_NUM_ITEMS; i++)
     {
         ptr_item_name = buflist_bar_item_get_name (i);
@@ -295,7 +298,7 @@ buflist_bar_item_auto_scroll (const char *item_name, int line_number)
 }
 
 /*
- * Returns content of bar item "buffer_plugin": bar item with buffer plugin.
+ * Returns the content of the bar item.
  */
 
 char *
@@ -369,8 +372,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
     snprintf (str_format_number_empty, sizeof (str_format_number_empty),
               "%%-%ds", length_max_number);
 
-    if (buflist_list_buffers[item_index])
-        weechat_arraylist_free (buflist_list_buffers[item_index]);
+    weechat_arraylist_free (buflist_list_buffers[item_index]);
     buflist_list_buffers[item_index] = weechat_arraylist_new (
         16, 0, 1,
         NULL, NULL, NULL, NULL);
@@ -566,7 +568,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
                                                    str_hotlist_count);
                     if (count > 0)
                     {
-                        if (*hotlist[0])
+                        if ((*hotlist)[0])
                         {
                             weechat_string_dyn_concat (
                                 hotlist,
@@ -596,8 +598,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
         weechat_hashtable_set (buflist_hashtable_extra_vars,
                                "hotlist",
                                (str_hotlist) ? str_hotlist : "");
-        if (str_hotlist)
-            free (str_hotlist);
+        free (str_hotlist);
 
         /* lag */
         ptr_lag = weechat_buffer_get_string (ptr_buffer, "localvar_lag");
@@ -629,8 +630,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
             buflist_hashtable_extra_vars,
             buflist_hashtable_options_conditions);
         rc = (condition && (strcmp (condition, "1") == 0));
-        if (condition)
-            free (condition);
+        free (condition);
         if (!rc)
             continue;
 
@@ -644,7 +644,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
 
         /* add newline between each buffer (if needed) */
         if (weechat_config_boolean (buflist_config_look_add_newline)
-            && *buflist[0])
+            && (*buflist)[0])
         {
             if (!weechat_string_dyn_concat (buflist, "\n", -1))
                 goto error;
@@ -765,6 +765,7 @@ buflist_bar_item_end ()
     for (i = 0; i < BUFLIST_BAR_NUM_ITEMS; i++)
     {
         weechat_bar_item_remove (buflist_bar_item_buflist[i]);
+        buflist_bar_item_buflist[i] = NULL;
     }
 
     weechat_hashtable_free (buflist_hashtable_pointers);
