@@ -31,6 +31,15 @@
 #define NI_MAXHOST 256
 #endif /* NI_MAXHOST */
 
+enum t_irc_server_ipv6
+{
+    IRC_SERVER_IPV6_DISABLE = 0,
+    IRC_SERVER_IPV6_AUTO,
+    IRC_SERVER_IPV6_FORCE,
+    /* number of IPv6 options */
+    IRC_SERVER_NUM_IPV6,
+};
+
 enum t_irc_server_sasl_fail
 {
     IRC_SERVER_SASL_FAIL_CONTINUE = 0,
@@ -76,9 +85,10 @@ enum t_irc_server_option
     IRC_SERVER_OPTION_REALNAME,      /* real name                            */
     IRC_SERVER_OPTION_LOCAL_HOSTNAME,/* custom local hostname                */
     IRC_SERVER_OPTION_USERMODE,      /* usermode to set once connected       */
+    IRC_SERVER_OPTION_COMMAND_DELAY, /* delay before execution of command    */
     IRC_SERVER_OPTION_COMMAND,       /* command to run once connected        */
-    IRC_SERVER_OPTION_COMMAND_DELAY, /* delay after execution of command     */
-    IRC_SERVER_OPTION_AUTOJOIN,      /* channels to automatically join       */
+    IRC_SERVER_OPTION_AUTOJOIN_DELAY, /* delay before autojoin               */
+    IRC_SERVER_OPTION_AUTOJOIN,       /* channels to automatically join      */
     IRC_SERVER_OPTION_AUTOJOIN_DYNAMIC, /* auto set autojoin option          */
     IRC_SERVER_OPTION_AUTOREJOIN,    /* auto rejoin channels when kicked     */
     IRC_SERVER_OPTION_AUTOREJOIN_DELAY,     /* delay before auto rejoin      */
@@ -98,33 +108,17 @@ enum t_irc_server_option
     IRC_SERVER_NUM_OPTIONS,
 };
 
-#define IRC_SERVER_OPTION_BOOLEAN(__server, __index)                          \
-    ((!weechat_config_option_is_null(__server->options[__index])) ?           \
-     weechat_config_boolean(__server->options[__index]) :                     \
-     ((!weechat_config_option_is_null(irc_config_server_default[__index])) ?  \
-      weechat_config_boolean(irc_config_server_default[__index])              \
-      : weechat_config_boolean_default(irc_config_server_default[__index])))
+#define IRC_SERVER_OPTION_BOOLEAN(__server, __index)                    \
+    weechat_config_boolean_inherited (__server->options[__index])
 
-#define IRC_SERVER_OPTION_INTEGER(__server, __index)                          \
-    ((!weechat_config_option_is_null(__server->options[__index])) ?           \
-     weechat_config_integer(__server->options[__index]) :                     \
-     ((!weechat_config_option_is_null(irc_config_server_default[__index])) ?  \
-      weechat_config_integer(irc_config_server_default[__index])              \
-      : weechat_config_integer_default(irc_config_server_default[__index])))
+#define IRC_SERVER_OPTION_INTEGER(__server, __index)                    \
+    weechat_config_integer_inherited (__server->options[__index])
 
-#define IRC_SERVER_OPTION_STRING(__server, __index)                           \
-    ((!weechat_config_option_is_null(__server->options[__index])) ?           \
-     weechat_config_string(__server->options[__index]) :                      \
-     ((!weechat_config_option_is_null(irc_config_server_default[__index])) ?  \
-      weechat_config_string(irc_config_server_default[__index])               \
-      : weechat_config_string_default(irc_config_server_default[__index])))
+#define IRC_SERVER_OPTION_STRING(__server, __index)                     \
+    weechat_config_string_inherited (__server->options[__index])
 
-#define IRC_SERVER_OPTION_ENUM(__server, __index)                             \
-    ((!weechat_config_option_is_null(__server->options[__index])) ?           \
-     weechat_config_enum(__server->options[__index]) :                        \
-     ((!weechat_config_option_is_null(irc_config_server_default[__index])) ?  \
-      weechat_config_enum(irc_config_server_default[__index])                 \
-      : weechat_config_enum_default(irc_config_server_default[__index])))
+#define IRC_SERVER_OPTION_ENUM(__server, __index)                       \
+    weechat_config_enum_inherited (__server->options[__index])
 
 #define IRC_SERVER_DEFAULT_PORT_CLEARTEXT 6667
 #define IRC_SERVER_DEFAULT_PORT_TLS       6697
@@ -269,7 +263,9 @@ struct t_irc_server
     int reconnect_delay;            /* current reconnect delay (growing)     */
     time_t reconnect_start;         /* this time + delay = reconnect time    */
     time_t command_time;            /* this time + command_delay = time to   */
-                                    /* autojoin channels                     */
+                                    /* execute command                       */
+    time_t autojoin_time;           /* this time + autojoin_delay = time to  */
+                                    /* auto-join channels                    */
     int autojoin_done;              /* 1 if autojoin has been done           */
     int disable_autojoin;           /* 1 if user asked to not autojoin chans */
     int is_away;                    /* 1 is user is marked as away           */
@@ -337,6 +333,7 @@ extern struct t_irc_server *irc_servers;
 extern const int gnutls_cert_type_prio[];
 extern const int gnutls_prot_prio[];
 extern struct t_irc_message *irc_recv_msgq, *irc_msgq_last_msg;
+extern char *irc_server_ipv6_string[];
 extern char *irc_server_sasl_fail_string[];
 extern char *irc_server_options[][2];
 
@@ -395,7 +392,8 @@ extern char *irc_server_get_default_msg (const char *default_msg,
                                          struct t_irc_server *server,
                                          const char *channel_name,
                                          const char *target_nick);
-extern void irc_server_buffer_set_input_multiline (struct t_irc_server *server,
+extern void irc_server_set_buffer_input_prompt (struct t_irc_server *server);
+extern void irc_server_set_buffer_input_multiline (struct t_irc_server *server,
                                                    int multiline);
 extern int irc_server_has_channels (struct t_irc_server *server);
 extern struct t_irc_server *irc_server_alloc (const char *name);
@@ -445,6 +443,7 @@ extern void irc_server_switch_address (struct t_irc_server *server,
 extern void irc_server_disconnect (struct t_irc_server *server,
                                    int switch_address, int reconnect);
 extern void irc_server_disconnect_all ();
+extern void irc_server_execute_command (struct t_irc_server *server);
 extern void irc_server_free_sasl_data (struct t_irc_server *server);
 extern void irc_server_free (struct t_irc_server *server);
 extern int irc_server_xfer_send_ready_cb (const void *pointer, void *data,
