@@ -1,7 +1,7 @@
 /*
  * weechat-perl.c - perl plugin for WeeChat
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2008 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -22,6 +22,16 @@
 
 #undef _
 
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+
+/* for wcwidth in wchar.h */
+#ifndef __USE_XOPEN
+#define __USE_XOPEN
+#endif
+#include <wchar.h>
+
 #include <locale.h>
 #include <EXTERN.h>
 #include <perl.h>
@@ -229,7 +239,7 @@ weechat_perl_output_flush ()
     char *temp_buffer, *command;
     int length;
 
-    if (!*perl_buffer_output[0])
+    if (!(*perl_buffer_output)[0])
         return;
 
     /* if there's no buffer, we catch the output, so there's no flush */
@@ -366,16 +376,16 @@ weechat_perl_exec (struct t_plugin_script *script,
             {
                 case 's': /* string or null */
                     if (argv[i])
-                        XPUSHs (sv_2mortal(newSVpv((char *)argv[i], 0)));
+                        XPUSHs (sv_2mortal (newSVpv((char *)argv[i], 0)));
                     else
-                        XPUSHs (sv_2mortal(&PL_sv_undef));
+                        XPUSHs (sv_2mortal (&PL_sv_undef));
                     break;
                 case 'i': /* integer */
-                    XPUSHs (sv_2mortal(newSViv(*((int *)argv[i]))));
+                    XPUSHs (sv_2mortal (newSViv (*((int *)argv[i]))));
                     break;
                 case 'h': /* hash */
                     hash = weechat_perl_hashtable_to_hash (argv[i]);
-                    XPUSHs (sv_2mortal(newRV_inc((SV *)hash)));
+                    XPUSHs (sv_2mortal (newRV_inc ((SV *)hash)));
                     break;
             }
         }
@@ -507,7 +517,9 @@ weechat_perl_load (const char *filename, const char *code)
     struct stat buf;
     char *perl_code;
     int length;
-#ifndef MULTIPLICITY
+#ifdef MULTIPLICITY
+    int wcwidth160;
+#else
     char pkgname[64];
 #endif /* MULTIPLICITY */
 
@@ -564,14 +576,22 @@ weechat_perl_load (const char *filename, const char *code)
               PERL_PLUGIN_NAME);
 
     PERL_SET_CONTEXT (perl_current_interpreter);
+    wcwidth160 = wcwidth (160);
     perl_construct (perl_current_interpreter);
+
+#if PERL_REVISION >= 6 || (PERL_REVISION == 5 && PERL_VERSION >= 28) || (PERL_REVISION == 5 && PERL_VERSION == 27 && PERL_SUBVERSION >= 9)
+    if (wcwidth (160) != wcwidth160)
+    {
+        /* restore the locale that's broken in some versions of Perl */
+        Perl_setlocale (LC_ALL, "");
+    }
+#else
+    (void) wcwidth160;
+#endif
+
     temp_script.interpreter = (PerlInterpreter *) perl_current_interpreter;
     perl_parse (perl_current_interpreter, weechat_perl_api_init,
                 perl_args_count, perl_args, NULL);
-#if PERL_REVISION >= 6 || (PERL_REVISION == 5 && PERL_VERSION >= 38)
-    /* restore the locale that could be changed by Perl >= 5.38 */
-    Perl_setlocale (LC_CTYPE, "");
-#endif
     length = strlen (perl_weechat_code) + strlen (str_warning) +
         strlen (str_error) - 2 + 4 + strlen ((code) ? code : filename) + 4 + 1;
     perl_code = malloc (length);
@@ -709,8 +729,7 @@ weechat_perl_unload (struct t_plugin_script *script)
                                        WEECHAT_SCRIPT_EXEC_INT,
                                        script->shutdown_func,
                                        NULL, NULL);
-        if (rc)
-            free (rc);
+        free (rc);
     }
 
     filename = strdup (script->filename);
@@ -736,14 +755,12 @@ weechat_perl_unload (struct t_plugin_script *script)
         PERL_SET_CONTEXT (perl_current_script->interpreter);
     }
 #else
-    if (interpreter)
-        free (interpreter);
+    free (interpreter);
 #endif /* MULTIPLICITY */
 
     (void) weechat_hook_signal_send ("perl_script_unloaded",
                                      WEECHAT_HOOK_SIGNAL_STRING, filename);
-    if (filename)
-        free (filename);
+    free (filename);
 }
 
 /*
@@ -835,13 +852,15 @@ weechat_perl_eval (struct t_gui_buffer *buffer, int send_to_buffer_as_input,
                    int exec_commands, const char *code)
 {
     void *func_argv[1], *result;
+    int old_perl_quiet;
 
     if (!perl_script_eval)
     {
+        old_perl_quiet = perl_quiet;
         perl_quiet = 1;
         perl_script_eval = weechat_perl_load (WEECHAT_SCRIPT_EVAL_NAME,
                                               PERL_EVAL_SCRIPT);
-        perl_quiet = 0;
+        perl_quiet = old_perl_quiet;
         if (!perl_script_eval)
             return 0;
     }
@@ -859,8 +878,7 @@ weechat_perl_eval (struct t_gui_buffer *buffer, int send_to_buffer_as_input,
                                 "script_perl_eval",
                                 "s", func_argv);
     /* result is ignored */
-    if (result)
-        free (result);
+    free (result);
 
     weechat_perl_output_flush ();
 
@@ -871,9 +889,10 @@ weechat_perl_eval (struct t_gui_buffer *buffer, int send_to_buffer_as_input,
 
     if (!weechat_config_boolean (perl_config_look_eval_keep_context))
     {
+        old_perl_quiet = perl_quiet;
         perl_quiet = 1;
         weechat_perl_unload (perl_script_eval);
-        perl_quiet = 0;
+        perl_quiet = old_perl_quiet;
         perl_script_eval = NULL;
     }
 
@@ -890,7 +909,7 @@ weechat_perl_command_cb (const void *pointer, void *data,
                          int argc, char **argv, char **argv_eol)
 {
     char *ptr_name, *ptr_code, *path_script;
-    int i, send_to_buffer_as_input, exec_commands;
+    int i, send_to_buffer_as_input, exec_commands, old_perl_quiet;
 
     /* make C compiler happy */
     (void) pointer;
@@ -949,6 +968,7 @@ weechat_perl_command_cb (const void *pointer, void *data,
                  || (weechat_strcmp (argv[1], "reload") == 0)
                  || (weechat_strcmp (argv[1], "unload") == 0))
         {
+            old_perl_quiet = perl_quiet;
             ptr_name = argv_eol[2];
             if (strncmp (ptr_name, "-q ", 3) == 0)
             {
@@ -966,8 +986,7 @@ weechat_perl_command_cb (const void *pointer, void *data,
                                                          ptr_name, 1);
                 weechat_perl_load ((path_script) ? path_script : ptr_name,
                                    NULL);
-                if (path_script)
-                    free (path_script);
+                free (path_script);
             }
             else if (weechat_strcmp (argv[1], "reload") == 0)
             {
@@ -979,7 +998,7 @@ weechat_perl_command_cb (const void *pointer, void *data,
                 /* unload perl script */
                 weechat_perl_unload_name (ptr_name);
             }
-            perl_quiet = 0;
+            perl_quiet = old_perl_quiet;
         }
         else if (weechat_strcmp (argv[1], "eval") == 0)
         {
@@ -1248,6 +1267,11 @@ weechat_perl_signal_quit_upgrade_cb (const void *pointer, void *data,
 int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
+    int old_perl_quiet;
+#ifndef MULTIPLICITY
+    int wcwidth160;
+#endif /* MULTIPLICITY */
+
 #ifdef PERL_SYS_INIT3
     int a;
     char **perl_args_local;
@@ -1264,6 +1288,11 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     weechat_perl_plugin = plugin;
 
+    perl_quiet = 0;
+    perl_eval_mode = 0;
+    perl_eval_send_input = 0;
+    perl_eval_exec_commands = 0;
+
     /* set interpreter name and version */
     weechat_hashtable_set (plugin->variables, "interpreter_name",
                            plugin->name);
@@ -1292,13 +1321,21 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
         return WEECHAT_RC_ERROR;
     }
 
+    wcwidth160 = wcwidth (160);
     perl_construct (perl_main);
+
+#if PERL_REVISION >= 6 || (PERL_REVISION == 5 && PERL_VERSION >= 28) || (PERL_REVISION == 5 && PERL_VERSION == 27 && PERL_SUBVERSION >= 9)
+    if (wcwidth (160) != wcwidth160)
+    {
+        /* restore the locale that's broken in some versions of Perl */
+        Perl_setlocale (LC_ALL, "");
+    }
+#else
+    (void) wcwidth160;
+#endif
+
     perl_parse (perl_main, weechat_perl_api_init, perl_args_count,
                 perl_args, NULL);
-#if PERL_REVISION >= 6 || (PERL_REVISION == 5 && PERL_VERSION >= 38)
-    /* restore the locale that could be changed by Perl >= 5.38 */
-    Perl_setlocale (LC_CTYPE, "");
-#endif
 #endif /* MULTIPLICITY */
 
     perl_data.config_file = &perl_config_file;
@@ -1314,11 +1351,13 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     perl_data.callback_signal_debug_dump = &weechat_perl_signal_debug_dump_cb;
     perl_data.callback_signal_script_action = &weechat_perl_signal_script_action_cb;
     perl_data.callback_load_file = &weechat_perl_load_cb;
+    perl_data.init_before_autoload = NULL;
     perl_data.unload_all = &weechat_perl_unload_all;
 
+    old_perl_quiet = perl_quiet;
     perl_quiet = 1;
     plugin_script_init (weechat_perl_plugin, &perl_data);
-    perl_quiet = 0;
+    perl_quiet = old_perl_quiet;
 
     plugin_script_display_short_list (weechat_perl_plugin,
                                       perl_scripts);
@@ -1337,7 +1376,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
+    int old_perl_quiet;
+
     /* unload all scripts */
+    old_perl_quiet = perl_quiet;
     perl_quiet = 1;
     if (perl_script_eval)
     {
@@ -1345,7 +1387,7 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
         perl_script_eval = NULL;
     }
     plugin_script_end (plugin, &perl_data);
-    perl_quiet = 0;
+    perl_quiet = old_perl_quiet;
 
 #ifndef MULTIPLICITY
     /* free perl interpreter */
@@ -1368,12 +1410,22 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
 
     /* free some data */
     if (perl_action_install_list)
+    {
         free (perl_action_install_list);
+        perl_action_install_list = NULL;
+    }
     if (perl_action_remove_list)
+    {
         free (perl_action_remove_list);
+        perl_action_remove_list = NULL;
+    }
     if (perl_action_autoload_list)
+    {
         free (perl_action_autoload_list);
+        perl_action_autoload_list = NULL;
+    }
     weechat_string_dyn_free (perl_buffer_output, 1);
+    perl_buffer_output = NULL;
 
     return WEECHAT_RC_OK;
 }
