@@ -719,7 +719,6 @@ spell_modifier_cb (const void *pointer, void *data,
                    const char *modifier,
                    const char *modifier_data, const char *string)
 {
-    unsigned long value;
     struct t_gui_buffer *buffer;
     struct t_spell_speller_buffer *ptr_speller_buffer;
     char **result, *ptr_string, *ptr_string_orig, *pos_space;
@@ -742,12 +741,10 @@ spell_modifier_cb (const void *pointer, void *data,
     if (!string)
         return NULL;
 
-    rc = sscanf (modifier_data, "%lx", &value);
+    rc = sscanf (modifier_data, "%p", &buffer);
     if ((rc == EOF) || (rc == 0))
         return NULL;
 
-    buffer = (struct t_gui_buffer *)value;
-
     /* check text during search only if option is enabled */
     if (weechat_buffer_get_integer (buffer, "text_search")
         && !weechat_config_boolean (spell_config_check_during_search))
