@@ -31,8 +31,8 @@ extern "C"
 #include <string.h>
 #include <time.h>
 #include <sys/time.h>
-#include "src/core/wee-string.h"
-#include "src/core/wee-util.h"
+#include "src/core/core-string.h"
+#include "src/core/core-util.h"
 }
 
 #define WEE_PARSE_DATE(__result, __sec, __usec, __datetime)    \
@@ -236,6 +236,14 @@ TEST(CoreUtil, Strftimeval)
     LONGS_EQUAL(20, util_strftimeval (str_time, sizeof (str_time),
                                       "%Y-%m-%d %H:%M:%S.%.7", &tv));
     STRCMP_EQUAL("2023-12-25 10:29:09.", str_time);
+
+    /* timestamp */
+    strcpy (str_time, "test");
+    LONGS_EQUAL(10, util_strftimeval (str_time, sizeof (str_time), "%!", &tv));
+    STRCMP_EQUAL("1703500149", str_time);
+    strcpy (str_time, "test");
+    LONGS_EQUAL(17, util_strftimeval (str_time, sizeof (str_time), "%!.%f", &tv));
+    STRCMP_EQUAL("1703500149.456789", str_time);
 }
 
 /*
@@ -480,6 +488,10 @@ TEST(CoreUtil, ParseDelay)
 
 TEST(CoreUtil, VersionNumber)
 {
+    LONGS_EQUAL(0, util_version_number (NULL));
+    LONGS_EQUAL(0, util_version_number (""));
+    LONGS_EQUAL(0, util_version_number ("abc"));
+
     LONGS_EQUAL(0x00030200, util_version_number ("0.3.2-dev"));
     LONGS_EQUAL(0x00030200, util_version_number ("0.3.2-rc1"));
     LONGS_EQUAL(0x00030200, util_version_number ("0.3.2"));
